"use strict";
exports.__esModule = true;
exports.setPropertyValueSpaces = exports.canReplaceSpacesOrTabs = exports.FormatProperty = void 0;
var suf_regex_1 = require("suf-regex");
var utility_1 = require("../utility");
var format_utility_1 = require("./format.utility");
var format_convert_1 = require("./format.convert");
var logger_1 = require("../logger");
function FormatProperty(line, STATE) {
    var convert = false;
    var replaceSpaceOrTabs = false;
    var edit = line.get();
    var isComment = suf_regex_1.isComment(line.get());
    var SetPropertySpaceRes = setPropertyValueSpaces(STATE, line.get());
    line.set(SetPropertySpaceRes);
    if (utility_1.convertLine(line, STATE)) {
        var convertRes = format_convert_1.convertScssOrCss(line.get(), STATE);
        line.set(convertRes.text);
        convert = true;
    }
    // Set Context Vars
    STATE.CONTEXT.convert.wasLastLineCss = convert;
    var move = STATE.LOCAL_CONTEXT.indentation.offset !== 0 && !isComment;
    if (!move && canReplaceSpacesOrTabs(STATE, line.get())) {
        line.set(utility_1.replaceSpacesOrTabs(line.get(), STATE).trimRight());
        replaceSpaceOrTabs = true;
    }
    // Return
    if (move) {
        var offset = STATE.LOCAL_CONTEXT.indentation.offset;
        var distance = STATE.LOCAL_CONTEXT.indentation.distance;
        if (STATE.CONTEXT.wasLastHeaderIncludeMixin) {
            if (distance >= STATE.CONTEXT.tabs - STATE.CONFIG.tabSize) {
                offset = utility_1.getBlockHeaderOffset(distance, STATE.CONFIG.tabSize, STATE.CONTEXT.tabs, false);
            }
            else {
                offset = (STATE.CONTEXT.tabs - STATE.CONFIG.tabSize) - distance;
                STATE.CONTEXT.wasLastHeaderIncludeMixin = false;
                STATE.CONTEXT.tabs = STATE.CONTEXT.tabs - STATE.CONFIG.tabSize;
            }
        }
        edit = utility_1.replaceWithOffset(line.get(), offset, STATE).trimRight();
        logger_1.PushDebugInfo({
            title: 'PROPERTY: MOVE',
            lineNumber: STATE.currentLine,
            oldLineText: STATE.lines[STATE.currentLine],
            newLineText: edit,
            debug: STATE.CONFIG.debug,
            offset: offset,
            originalOffset: STATE.LOCAL_CONTEXT.indentation.offset,
            replaceSpaceOrTabs: replaceSpaceOrTabs
        });
    }
    else if (STATE.CONFIG.deleteWhitespace &&
        suf_regex_1.getDistanceReversed(line.get(), STATE.CONFIG.tabSize) > 0) {
        edit = line.get().trimRight();
        logger_1.PushDebugInfo({
            title: 'PROPERTY: TRAIL',
            lineNumber: STATE.currentLine,
            oldLineText: STATE.lines[STATE.currentLine],
            newLineText: edit,
            debug: STATE.CONFIG.debug,
            replaceSpaceOrTabs: replaceSpaceOrTabs
        });
    }
    else {
        edit = line.get();
        logger_1.PushDebugInfo({
            title: 'PROPERTY: DEFAULT',
            lineNumber: STATE.currentLine,
            oldLineText: STATE.lines[STATE.currentLine],
            newLineText: edit,
            debug: STATE.CONFIG.debug,
            replaceSpaceOrTabs: replaceSpaceOrTabs
        });
    }
    format_utility_1.FormatSetTabs(STATE);
    return edit;
}
exports.FormatProperty = FormatProperty;
function canReplaceSpacesOrTabs(STATE, text) {
    return STATE.CONFIG.insertSpaces
        ? /\t/g.test(text)
        : new RegExp(' '.repeat(STATE.CONFIG.tabSize), 'g').test(text);
}
exports.canReplaceSpacesOrTabs = canReplaceSpacesOrTabs;
function setPropertyValueSpaces(STATE, text) {
    if (text &&
        (!STATE.LOCAL_CONTEXT.isHtmlTag &&
            (STATE.LOCAL_CONTEXT.isProp || STATE.LOCAL_CONTEXT.isInterpolatedProp) &&
            STATE.CONFIG.setPropertySpace)) {
        var newPropValue = '';
        var _a = text.split(/:(.*)/), propName = _a[0], propValue = _a[1];
        var wasLastCharSpace = true;
        for (var i = 0; i < propValue.length; i++) {
            var char = propValue[i];
            switch (char) {
                case ' ':
                    if (!wasLastCharSpace) {
                        newPropValue += char;
                        wasLastCharSpace = true;
                    }
                    break;
                case '.':
                    wasLastCharSpace = true;
                    newPropValue += char;
                    break;
                default:
                    wasLastCharSpace = false;
                    newPropValue += char;
                    break;
            }
        }
        return propName.trimEnd() + ":" + (propValue ? ' ' + newPropValue : '');
    }
    return text;
}
exports.setPropertyValueSpaces = setPropertyValueSpaces;
